using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class PhysStick : MonoBehaviour
{
    float fixedLength;
    public PhysParticle pa;
    public PhysParticle pb;

    // Start is called before the first frame update
    void Start()
    {
        fixedLength = (pa.transform.position - pb.transform.position).magnitude;
    }

    // JUST FOR THE LOOK:
    // before every rendering (Update method),
    // we teleport/resize/reorient this stick (a cylinder)
    // so to visually connect the two particles
    void Update()
    {
        Vector3 posA = pa.transform.position;
        Vector3 posB = pb.transform.position;
        
        // note: position or localPosition is the same (the scene-graph is only 1 lvl deep)
        transform.localPosition = (posA + posB) * 0.5f;
        transform.localScale = new Vector3(0.1f, Vector3.Distance(posA, posB)*0.5f, 0.1f);
        transform.localRotation = Quaternion.FromToRotation(Vector3.up, posB - posA);
    }

    void enforcePositionalConstraint()
    {
        Vector3 posA = pa.transform.position;
        Vector3 posB = pb.transform.position;

        // "dear particles A and B: please be at distance fixedLength " 

        // let's apply the formulas we wrote at the blackboard,
        // to fullfill the constraint above (in quotes)
        // while minimizing the sum, over pA and pA, of mass x displacement^2 
        float mA = pa.mass;
        float mB = pb.mass;

        float currentLength = (posB - posA).magnitude;
        float deltaLength = fixedLength - currentLength;
        Vector3 dir = (posB - posA).normalized; // Versor form A toward B

        float kA = -mB / (mA + mB) * deltaLength;
        float kB = +mA / (mA + mB) * deltaLength;

        pa.transform.position += dir * kA;
        pb.transform.position += dir * kB;
    }

    // FixedUpdate is called once per physic frame
    void FixedUpdate()
    {
        enforcePositionalConstraint();
    }
}
